#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wmul_rivendell']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['wmul_logger>=0.5.1',
 'wmul_emailer>=0.5.0',
 'click',
 'cachetools',
 'numpy',
 'pandas',
 'openpyxl']

extras_require = \
{":python_version < '3.7.0'": ['mysql-connector-python<8.0.29',
                               'protobuf<=3.19.4',
                               'dataclasses'],
 ":python_version > '3.7.0'": ['mysql-connector-python>8.0',
                               'protobuf>=3.19.0'],
 'test': ['pytest', 'pytest-mock', 'pyfakefs', 'wmul_test_utils>=0.3.1'],
 'tox': ['tox']}

entry_points = \
{'console_scripts': ['wmul_rivendell = wmul_rivendell.cli:wmul_rivendell_cli']}

setup(name='wmul_rivendell',
      version='0.13.1',
      description='Various scripts to help use Rivendell Radio Automation 3.4.1+',
      author=None,
      author_email='Michael Stanley <stanley50@marshall.edu>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
     )
